// Draw
// Uses a linked list to store a list of points to draw
// Draws them using GDI+

#include "stdafx.h"

#using <mscorlib.dll>
#using <System.dll>
#using <System.Windows.Forms.dll> 
#using <System.Drawing.dll> 

using namespace System;
using namespace System::Drawing; 
using namespace System::Windows::Forms; 


//Stores point information
//Uses a linked list
class PointList 
{
public:
   int nX;
   int nY;
   PointList  *poNext;
};

//Gets the values for a new point
void NewPoint(PointList *poNew)
{
	//Read the x and y positions and store them in the structure
	Console::WriteLine(S"What is the x position?");
	poNew->nX = Int32::Parse(Console::ReadLine());
	Console::WriteLine(S"What is the y position?");
	poNew->nY = Int32::Parse(Console::ReadLine());
	//Set the pointer to null
	poNew->poNext = 0;
}

//Draws the points
void DrawPoint(Graphics *poGraphics, PointList *poPoint)
{
	//Is there a place to draw to?
	if (poPoint->poNext)
	{
		//Create a pen with which to draw
		Pen *poPen = new Pen(Color::Red);
		//Draw a line between the current point and the next point
		poGraphics->DrawLine(poPen, poPoint->nX, poPoint->nY, poPoint->poNext->nX, poPoint->poNext->nY); 
	}

}

// This is the entry point for this application
#ifdef _UNICODE
int wmain(void)
#else
int main(void)
#endif
{
	//Points to the list of coordinates
	PointList *poPoints = 0;
	//Points to the last PointList created
	PointList *poLast;
	//Points to a new PointList
	PointList *poNew;
	//For storing the user response
	String *pszMore;

	//Structures for drawing graphics
	Form *poForm = new Form(); 
	Graphics *poGraphics = poForm->CreateGraphics(); 


	//A flag to tell us to stop
	bool fFinished = false;

	while (!fFinished)
	{
		//Create a new point structure
		poNew = new PointList;
		if (!poPoints)
		{
			//If it is the first one, initialize poPoints
			poPoints = poNew;
		}
		else
		{
			//Connect it to the last one
			poLast->poNext = poNew;
		}
		//Now update poLast
		poLast = poNew;

		//Read in the values
		NewPoint(poNew);

		//Should we read more in?
		Console::WriteLine("Hit y to enter a new point");
		pszMore = Console::ReadLine();
		if (!pszMore->Equals(S"y"))
		{
			fFinished = true;
		}

	}

	//Show the display surface
	poForm->Show();

	//Now draw the points
	poLast = poPoints;
	while (poLast)
	{
		//Draw it
		DrawPoint(poGraphics, poLast);
		//Advance
		poLast = poLast->poNext;
	}

	//Free up the graphics surface
	poGraphics->Dispose();



	//Now clean up the memory
	poLast = poPoints;
	while (poLast)
	{
		PointList *poNext;
		poNext = poLast->poNext;

		//Free up the memory
		delete poLast;

		//Now advance
		poLast = poNext;
	}
	

	//Hang out until the user closes the form
	Application::Run(poForm);

}