// Draw5
// Introduces class structures
// Unmanaged

#include "stdafx.h"
#include <iostream.h>

//Stores line information
class LineObject 
{
public:
	void Initialize();
	void Draw();
private:
   int m_nXFrom;
   int m_nYFrom;
   int m_nXTo;
   int m_nYTo;
};

//Initialize values for this line object
void LineObject::Initialize()
{
	//Read the x and y positions and store them in the structure
	cout << "What is the starting x position?";
	cin >> m_nXFrom;
	cout << "What is the starting y position?";
	cin >> m_nYFrom;
	cout << "What is the ending x position?";
	cin >> m_nXTo;
	cout << "What is the ending y position?";
	cin >> m_nYTo;
}

//Draws the line. We'll write out the contents
void LineObject::Draw()
{
	cout << "From " << m_nXFrom << "," << m_nYFrom << " to " << m_nXTo << "," << m_nYTo << endl;
}

//Class for constructing a linked list
class LineObjectContainer
{
public:
	LineObject *poLine;
	LineObjectContainer *poNext;
	void Delete();
};

//Clean up the list
void LineObjectContainer::Delete()
{
	LineObjectContainer *poCur, *poTemp;

	poCur = this;
	while (poCur)
	{
		poTemp = poCur;
		//Clear out the line object
		delete poCur->poLine;
		poCur = poCur->poNext;
		//Delete the container object
		delete poTemp;
	}
}

//Stores a linked list of objects
class ArrayList
{
public:
	void Add(LineObject *poLine);
	LineObject *MoveNext();
	void Delete();
	void Initialize();
private:
	LineObjectContainer *m_poFirst;
	LineObjectContainer *m_poCur;
	LineObjectContainer *m_poLast;
};


//Initialize the structures
void ArrayList::Initialize()
{
	m_poFirst = 0;
	m_poCur = 0;
	m_poLast = 0;
}

//Adds a new object to the list
void ArrayList::Add(LineObject *poLine)
{
	//Create a new container
	LineObjectContainer *poLOC = new LineObjectContainer();
	//Set its value
	poLOC->poLine = poLine;
	poLOC->poNext = 0;

	//Tie it in
	if (!m_poFirst)
	{
		m_poFirst = poLOC;
		m_poCur = m_poFirst;
	}
	else
		m_poLast->poNext = poLOC;
	//Advance the end pointer
	m_poLast = poLOC;
}

//Returns the next LineObject item
LineObject *ArrayList::MoveNext()
{
	LineObject *poNext;
	
	//If there are no items, just return null
	if (!m_poCur)
		return 0;

	//Find the next line object
	poNext = m_poCur->poLine;
	//Advance the pointer
	m_poCur = m_poCur->poNext;
	return poNext;
}

//Clean up memory
void ArrayList::Delete()
{
	//Delete the list of line objects
	m_poFirst->Delete();
}

//Stores all of the lines to draw
class DisplayObject
{
public:
	void Add();
	void Draw();
	void Initialize();
	void Delete();
private:
	ArrayList *m_paLines;
};

//Initialize the DisplayObject structures
void DisplayObject::Initialize()
{
	m_paLines = new ArrayList();
	m_paLines->Initialize();
}

//Clean it up
void DisplayObject::Delete()
{
	m_paLines->Delete();
}

//Add a new object
void DisplayObject::Add()
{
	bool fFinished = false;
	char cMore;

	while (!fFinished)
	{
		//Create a new line object
		LineObject *poLine = new LineObject();
		//Initialize it
		poLine->Initialize();
		//Store it in the structure
		m_paLines->Add(poLine);

		//Should we add another one?
		cout << "Hit y to enter a new line" << endl;
		cin >> cMore;
		if (cMore != 'y')
		{
			fFinished = true;
		}
	}
}

//Draw all of the lines
void DisplayObject::Draw()
{
	LineObject *poLine;

	//Go through each of the line objects
    while (poLine = m_paLines->MoveNext())
	{
		//Call the draw method for the object
		poLine->Draw();
	}
}


int _tmain(int argc, _TCHAR* argv[])
{
	//Structure for the display
	DisplayObject *poDisplay = new DisplayObject();

	//Initialize the display object
	poDisplay->Initialize();

	//Add the lines
	poDisplay->Add();

	//Draw the lines
	poDisplay->Draw();

	//Clean up
	poDisplay->Delete();
	delete poDisplay;

	return 0;
}

