// Draw8
// Uses inheritance and virtual functions


#include "stdafx.h"

#using <mscorlib.dll>
#using <System.dll>
#using <System.Windows.Forms.dll> 
#using <System.Drawing.dll> 

using namespace System;
using namespace System::Drawing; 
using namespace System::Windows::Forms; 
using namespace System::Collections;

//Stores shape information
//An abstract base class
__gc class ShapeObject
{
public:
	virtual void Initialize() = 0;
	virtual void Draw(Graphics *poG) = 0;
};

//Stores line information
__gc class LineObject : public ShapeObject
{
public:
	virtual void Initialize();
	virtual void Draw(Graphics *poG);
private:
   int m_nXFrom;
   int m_nYFrom;
   int m_nXTo;
   int m_nYTo;
};

//Initialize values for this line object
void LineObject::Initialize()
{
	//Read the x and y positions and store them in the structure
	Console::WriteLine(S"What is the starting x position?");
	m_nXFrom = Int32::Parse(Console::ReadLine());
	Console::WriteLine(S"What is the starting y position?");
	m_nYFrom = Int32::Parse(Console::ReadLine());
	Console::WriteLine(S"What is the ending x position?");
	m_nXTo = Int32::Parse(Console::ReadLine());
	Console::WriteLine(S"What is the ending y position?");
	m_nYTo = Int32::Parse(Console::ReadLine());
}

//Draws the line
void LineObject::Draw(Graphics *poG)
{
	//Create a pen with which to draw
	Pen *poPen = new Pen(Color::Red);
	//Draw the line
	poG->DrawLine(poPen, m_nXFrom, m_nYFrom, m_nXTo, m_nYTo); 
}

//Stores circle information
__gc class CircleObject : public ShapeObject
{
public:
	virtual void Initialize();
	virtual void Draw(Graphics *poG);
private:
   int m_nXCenter;
   int m_nYCenter;
   int m_nRadius;
};

//Initialize values for this circle object
void CircleObject::Initialize()
{
	//Read the center position and radius and store them in the structure
	Console::WriteLine(S"What is the center x position?");
	m_nXCenter = Int32::Parse(Console::ReadLine());
	Console::WriteLine(S"What is the center y position?");
	m_nYCenter = Int32::Parse(Console::ReadLine());
	Console::WriteLine(S"What is the radius?");
	m_nRadius = Int32::Parse(Console::ReadLine());
}

//Draws the line
void CircleObject::Draw(Graphics *poG)
{
	//Create a pen with which to draw
	Pen *poPen = new Pen(Color::Blue);
	//Draw the line
	poG->DrawEllipse(poPen, m_nXCenter-m_nRadius, m_nYCenter-m_nRadius, m_nXCenter+m_nRadius, m_nYCenter+m_nRadius); 
}

//Stores all of the lines to draw
__gc class DisplayObject
{
public:
	void Add();
	void Draw(Graphics *poG);
	DisplayObject();
private:
	ArrayList *m_paShapes;
};

//Initialize the DisplayObject structures
DisplayObject::DisplayObject()
{
	m_paShapes = new ArrayList();
}

//Add a new object
void DisplayObject::Add()
{
	bool fFinished = false;
	String *pszMore;
	ShapeObject *poShape;

	while (!fFinished)
	{
		//What shall we create?
		Console::WriteLine("Type c to create a circle and l to create a line");
		pszMore = Console::ReadLine();

		if (pszMore->Equals(S"c"))
		{
			//We'll create a circle
			poShape = new CircleObject();
		}
		else
		{
			//We'll create a line
			poShape = new LineObject();
		}

		//Initialize it
		poShape->Initialize();
		//Store it in the structure
		m_paShapes->Add(poShape);

		//Should we add another one?
		Console::WriteLine("Hit y to enter a new shape");
		pszMore = Console::ReadLine();
		if (!pszMore->Equals(S"y"))
		{
			fFinished = true;
		}
	}
}

//Draw all of the shapes
void DisplayObject::Draw(Graphics *poG)
{
	//Go through each of the shape objects
    IEnumerator *poEnumerator = m_paShapes->GetEnumerator();
    while (poEnumerator->MoveNext())
	{
		//Call the draw method for the object
		static_cast<ShapeObject*>(poEnumerator->Current)->Draw(poG);
	}
}

// This is the entry point for this application
#ifdef _UNICODE
int wmain(void)
#else
int main(void)
#endif
{
	//Structures for drawing graphics
	Form *poForm = new Form(); 
	Graphics *poGraphics = poForm->CreateGraphics(); 
	DisplayObject *poDisplay = new DisplayObject();


	//Add the shapes
	poDisplay->Add();

	//Move the display surface to the far left
	poForm->SetDesktopLocation(0,0);
	//Show the display surface
	poForm->Show();


	//Draw the shapes
	poDisplay->Draw(poGraphics);

	//Free up the graphics surface
	poGraphics->Dispose();

	//Hang out until the user closes the form
	Application::Run(poForm);
}